*********************************************************************
*     PROGRAM psfc4phits                                            *
*                                                                   *
*         Created by S. Hashimoto on 2015/02/13                     *
*             based on test_IAEAphsp_f.F distributed by IAEA        *
*                                                                   *
*-------------------------------------------------------------------*
*                                                                   *
*     Purpose                                                       *
*         Convert IAEA Phase Space File to PHITS dumpfile           *
*     Required files to compile                                     *
*         iaea_config.h, iaea_header.h, iaea_phsp.h, iaea_record.h, *
*         utilities.h, iaea_header.cpp, iaea_phsp.cpp,              *
*         iaea_record.cpp, utilities.cpp                            *
*     Compile                                                       *
*         Windows: Double click "compile_win.bat"                   *
*         Mac: Type ./compile_mac.sh in this directory              *
*         Linux: Type ./compile_lin.sh in this directory            *
*                                                                   *
*         Last modified by S. Hashimoto on 2016/05/26               *
*********************************************************************

#ifdef DOUBLE
#define IAEA_Float real*8
#else
#define IAEA_Float real*4
#endif

#define IAEA_I16 integer*2
#define IAEA_I32 integer*4
#define IAEA_I64 integer*8

#define NUM_EXTRA_LONG 1
#define NUM_EXTRA_FLOAT 1

      program test

      character*80  file_name
      integer*4     i
      IAEA_I32      source_read,source_write,access_read,access_write,
     &              result
      parameter    (access_read=1,access_write=2)
      IAEA_I32      typ,extra_ints(NUM_EXTRA_LONG),n_stat,
     &              iextrafloat,iextralong,ind,extralong_type
      IAEA_Float    E, wt, x, y, z, u, v, w, z_constant, 
     &              extra_floats(NUM_EXTRA_FLOAT)
      IAEA_I64      histories,nphot,irecord,read_indep,nrecorded,norig
C S.Hashimoto
      character*80  dmpfile_PHITS
      integer iopt
C S.Hashimoto

      do i=1,len(file_name)
          file_name(i:i) = char(0)
      end do

      !
      ! ***** Create a new source for reading
      !       Because file_name is empty, iaea_new_source will ask for 
      !       a name
      !
      call iaea_new_source(source_read,file_name,access_read,result)
      if( result.lt.0 ) then
          write(6,*) 'Error creating new source for reading particles'
          call exit(1)
      end if
      
      !
      ! ***** Get the total number of particles in the file
      !
      typ = -1  ! could be 0
      call iaea_get_max_particles(source_read, typ, histories)
      write(6,*) 'Total number of particles: ',histories
      !
      ! ***** Get the number of photons in the file
      !
      typ = 1  
      call iaea_get_max_particles(source_read, typ, nphot)
      write(6,*) 'Number of photons: ',nphot
      !
      ! ***** Get the number of original statistically independent particles
      !
      call iaea_get_total_original_particles(source_read,norig)
      write(6,*) 'Original particles: ',norig

      do i=1,len(file_name)
          file_name(i:i) = char(0)
      end do
      
      !
      ! ***** Create a new source for writing
      !       Because file_name is empty, iaea_new_source will ask for 
      !       a name
      !
      call iaea_new_source(source_write,file_name,access_write,result)
      if( result.lt.0 ) then
          write(6,*) 'Error creating new source for writing particles'
          call exit(1)
      end if
      !
      ! ***** Set the number of extra variables to be stored:
      !       0 extra floats and 1 extra long
      !
      iextrafloat = 0
      iextralong = 1
      call iaea_set_extra_numbers(source_write,iextrafloat,iextralong)
      !
      ! ***** Set the type of first extra long being stored
      !
      ind = 0
      extralong_type = 2  ! LATCH
      call iaea_set_type_extralong_variable(source_write,ind,
     &                                      extralong_type)
      !
      ! ***** Set z as a constant (i.e. it will not be stored in the file)
      !
      ind = 2
      z_constant = 80
      call iaea_set_constant_variable(source_write,ind,z_constant)
      
      !
      ! ***** Copy the header of the read source to the write source
      !
      call iaea_copy_header(source_read,source_write,result)

      !
      ! ***** Read particles from the read source and write them in the 
      !       write source
      !

      nrecorded = 0
C S.Hashimoto
      iunitt = 99
      iunito = 100
      iopt = 0
      open(iunitt,file='tmp.inp', status='old')
      read(iunitt,*)
      read(iunitt,*)
      read(iunitt,*) dmpfile_PHITS
      read(iunitt,*) nini
      read(iunitt,*) nfin
      read(iunitt,*) iopt
      close(iunitt)
      if (nini .gt. nfin) then
         write(*,*) 'Error: initial line > final line',nini,nfin
         stop
      end if

      if (iopt .lt. 0) then
         open(iunito,file=dmpfile_PHITS
     &        ,form='formatted',status = 'unknown')

      else if (iopt .gt. 0) then
         open(iunito,file=dmpfile_PHITS
     &        ,form='unformatted',status = 'unknown')
      else if (iopt .eq. 0) then
         open(unit=iunito,file=dmpfile_PHITS
     &        ,access='stream', status='unknown', iostat=ios)
      end if

      do irecord=1,histories
C S.Hashimoto
          !
          ! ***** read a particle
          !
          call iaea_get_particle(source_read,n_stat,typ,E,wt,x,y,z,
     &              u,v,w,extra_floats,extra_ints)
C S.Hashimoto
          if (typ.eq.1) then ! photons
             kfcode=22
          else if (typ.eq.2) then ! electrons
             kfcode=11
          else if (typ.eq.3) then ! positrons
             kfcode=-11
          else if (typ.eq.4) then ! neutrons
             kfcode=2112
          else if (typ.eq.5) then ! protons
             kfcode=2212
          else
             write(*,*) 'Warning: unexpected particle type=',typ
c$$$             stop
          end if

          if (irecord.ge.nini .and. irecord.le.nfin) then
             if (iopt .lt. 0) then
                write(iunito,'(9(1p1d24.15))')
     &               dble(kfcode),x,y,z,u,v,w,e,wt

             else if (iopt .ge. 0) then
                write(unit=iunito) dble(kfcode),dble(x),dble(y),dble(z)
     &               ,dble(u),dble(v),dble(w),dble(e),dble(wt)

             end if
          end if
C S.Hashimoto
          if( n_stat.eq.-1 ) exit  ! i.e. end of file
          !
          ! ***** write a particle
          !
c$$$          call iaea_write_particle(source_write,n_stat,typ,E,wt,x,y,z,
c$$$     &              u,v,w,extra_floats,extra_ints)
          nrecorded = nrecorded + 1
      end do

C S.Hashimoto
      close(iunito)
C S.Hashimoto

c$$$      call iaea_print_header(source_write,result)
c$$$
c$$$      write(6,*) 'Particles recorded: ',nrecorded

      !
      ! ***** Close sources
      !
      call iaea_destroy_source(source_read,result)
      call iaea_destroy_source(source_write,result)

      end
      
